/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
// CCaObject2.h : main header file for CCaObject2 class

#ifndef _INC_CCAOBJECT2_H
#define _INC_CCAOBJECT2_H

#define __USING_TIME_STAMP__

// Handle exports/imports
#ifdef WIN32
# ifdef CCAOBJECT2_EXPORTS
#  define LINK_DLL __declspec(dllexport)
# else
#  define LINK_DLL __declspec(dllimport)
# endif
#else
# define LINK_DLL
#endif

// Version
#define CAOBJECT2_VERSION        1
#define CAOBJECT2_REVISION       1
#define CAOBJECT2_MODIFICATION   0
#define CAOBJECT2_UPDATE_LEVEL   2
#define CAOBJECT2_VERSION_STRING "CaObject2 Version 1.1.0"

#include <stdarg.h>
#include <cadef.h>
#include <dbDefs.h>
#include <cvtFast.h>

#ifndef FLDNAME_SZ
#define FLDNAME_SZ 4  /*Field Name Size*/
#endif

// Should be defined for us (value is 34)
#define FULL_PVNAME_SZ PVNAME_SZ+FLDNAME_SZ+2

class LINK_DLL CCaObject2;

typedef enum {
    monitor_off,
    monitor_req,
    monitor_on
} ChanMonitorState;

typedef enum {
    COMPACT_STRING,
    DECIMAL_STRING,
    ENGR_NOTATION_STRING,
    EXPONENTIAL_STRING,
    HEXADECIMAL_STRING,
    OCTAL_STRING,
    STRING_STRING,
    TRUNCATED_STRING
} StringType;

typedef void (*CCaObject2AccessRightsCB)(CCaObject2 *chan);
typedef void (*CCaObject2ConnectionCB)(CCaObject2 *chan);
typedef void (*CCaObject2ExceptionCB)(struct exception_handler_args args);
typedef void (*CCaObject2GraphicalInfoCB)(CCaObject2 *chan);
typedef void (*CCaObject2ValueChangedCB)(CCaObject2 *chan);
typedef int (*CCaObject2ErrorHandler)(const char *fmt, ...);

#ifdef __USING_TIME_STAMP__
typedef union {
    struct dbr_time_string s;
    struct dbr_time_enum   e;
    struct dbr_time_char   c;
    struct dbr_time_short  i;
    struct dbr_time_long   l;
    struct dbr_time_float  f;
    struct dbr_time_double d;
} dataBuf;
#else
typedef union {
    struct dbr_sts_string s;
    struct dbr_sts_enum   e;
    struct dbr_sts_char   c;
    struct dbr_sts_short  i;
    struct dbr_sts_long   l;
    struct dbr_sts_float  f;
    struct dbr_sts_double d;
} dataBuf;
#endif

typedef union {
    struct dbr_sts_string  s;
    struct dbr_ctrl_enum   e;
    struct dbr_ctrl_char   c;
    struct dbr_ctrl_short  i;
    struct dbr_ctrl_long   l;
    struct dbr_ctrl_float  f;
    struct dbr_ctrl_double d;
} infoBuf;

class LINK_DLL CCaObject2 {
  public:
    CCaObject2();
    ~CCaObject2();
    int connect(const char *nameIn);
    void disconnect(void);
    void initialize(void);
    unsigned long getRefCount(void) const { return refCount; }
    int getConnected(void) const { return connected; }
    int startMonitor();
    int stopMonitor(void);
    int getMonitored(void) const { return monitored; }
    short getDataType(void) const { return dataType; }
    short getState(void) const { return ca_state(Chid); }
    int getElementCount(void) const {return elementCount; }
    double getValue(void) const {return value; }
    void *getArray(void) const { return array; }
    double getArrayValueAsDouble(int i=0) const;
    int getStringValue(char *string, int size, short prec,
      StringType type) const;
    double getHopr(void) const {return hopr; }
    double getLopr(void) const {return lopr; }
    short getPrecision(void) const {return precision; }
    short getStatus(void) const {return status; }
    short getSeverity(void) const {return severity; }
    char *getStateString(int i) const { return stateStrings[i]; }
    int getReadAccess(void) const {return readAccess; }
    int getWriteAccess(void) const {return writeAccess; }
    int getGraphicalInfoAvailable(void) const {return graphicalInfoAvailable; }
    const char *getName(void) const { return name; }
    chid getChid(void) const { return Chid; }
    TS_STAMP getTimeStamp(void) const { return time; };
    char *getTimeStampString(void);

    int getMonitorSeverityChanged(void) const { return monitorSeverityChanged; }
    void setMonitorSeverityChanged(int monitorSeverityChangedIn)
      { monitorSeverityChanged=monitorSeverityChangedIn; }
    int getmonitorValueChanged(void) const { return monitorValueChanged; }
    void setMonitorValueChanged(int monitorValueChangedIn)
      { monitorValueChanged=monitorValueChangedIn; }
    int getmonitorZeroTransition(void) const { return monitorZeroTransition; }
    void setMonitorZeroTransition(int monitorZeroTransitionIn)
      { monitorZeroTransition=monitorZeroTransitionIn; }

    void setAccessRightsCB(CCaObject2AccessRightsCB cb)
      {	accessRightsCB=cb; }
    void clearAccessRightsCB(void) { accessRightsCB=NULL; }
    CCaObject2AccessRightsCB getAccessRightsCB(void) const
      {	return accessRightsCB; }

    void setConnectionCB(CCaObject2ConnectionCB cb)
      {	connectionCB=cb; }
    void clearConnectionCB(void) { connectionCB=NULL; }
    CCaObject2ConnectionCB getConnectionCB(void) const
      {	return connectionCB; }

    void setExceptionCB(CCaObject2ExceptionCB cb)
      {	exceptionCB=cb; }
    void clearExceptionCB(void) { exceptionCB=NULL; }
    CCaObject2ExceptionCB getExceptionCB(void) const
      {	return exceptionCB; }
    CCaObject2ExceptionCB restoreExceptionCB(void)
      {	return (exceptionCB = defaultExceptionCB); }


    void setGraphicalInfoCB(CCaObject2GraphicalInfoCB cb)
      {	graphicalInfoCB=cb; }
    void clearGraphicalInfoCB(void) { graphicalInfoCB=NULL; }
    CCaObject2GraphicalInfoCB getGraphicalInfoCB(void) const
      {	return graphicalInfoCB; }

    void setValueChangedCB(CCaObject2ValueChangedCB cb)
      {	valueChangedCB=cb; }
    void clearValueChangedCB(void) { valueChangedCB=NULL; }
    CCaObject2ValueChangedCB getValueChangedCB(void) const
      {	return valueChangedCB; }

    void setErrorHandler(CCaObject2ErrorHandler handler)
      {	errHandler=handler;}
    void clearErrorHandler(void) { errHandler=NULL; }
    CCaObject2ErrorHandler getErrorHandler(void) const
      {	return errHandler; }
    CCaObject2ErrorHandler restoreErrorHandler(void)
      {	return (errHandler=errMsg); }

    void *getUserData(void) const { return userData; }
    void setUserData(void *userDataIn) { userData=userDataIn; }

    int getCallback(void);
    int getGraphicalInfoCallback(void);

    void putDouble(double val);
    void putLong(long lval);
    void putCharacterArray(char *chars, unsigned long size);
    void putString(char *string);

#ifdef todo
    int get(void);
    static void addFDRegistration(CAFDHANDLER *handler, void *pUser=NULL);
#endif

  private:
    int connected;     // If the class is connected, validly or not
    ChanMonitorState monitored;
    int elementCount;
    short dataType;
    double value;
    double hopr;
    double lopr;
    short precision;
    short status;
    short severity;
    int pvCurConnected;     // If the PV in the class is connected
    int pvPrevConnected;
    int readAccess;
    int writeAccess;
    int graphicalInfoAvailable;
    char *stateStrings[MAX_ENUM_STATES];
    char name[FULL_PVNAME_SZ];
    void *array;
    TS_STAMP time;
    int monitorSeverityChanged;
    int monitorValueChanged;
    int monitorZeroTransition;
    dataBuf *data;
    int dataSize;     // Size of data buffer in bytes
    infoBuf info;
    chid Chid;
    evid Evid;
    
    CCaObject2AccessRightsCB accessRightsCB;
    CCaObject2ConnectionCB connectionCB;
    CCaObject2GraphicalInfoCB graphicalInfoCB;
    CCaObject2ValueChangedCB valueChangedCB;

    static CCaObject2ExceptionCB exceptionCB;
    static CCaObject2ErrorHandler errHandler;

    static int caStarted;
    static unsigned long refCount;

    void *userData;

  private:
    int taskInit(void);
    int taskExit(void);

  public:
    static void accessRightsICB(struct access_rights_handler_args args);
    static void connectionICB(struct connection_handler_args args);
    static void exceptionICB(struct exception_handler_args args);
#ifdef todo
    static void fdRegistrationICB(void *dummy, int fd, int condition);
#endif
    static void graphicalInfoICB(struct event_handler_args args);
    static void valueChangedICB(struct event_handler_args args);

    static int errMsg(const char *fmt, ...);
    static void defaultExceptionCB(struct exception_handler_args args);

    static void poll(void) { ca_poll(); }
};

#endif // _INC_CCAOBJECT2_H
