/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
// Main include file for HistTool

#ifndef _INCLUDE_HISTTOOL_H
#define _INCLUDE_HISTTOOL_H

#define START_PVNAME "evans:calc"
#define INITFILENAME "histToolrc"
#define PROGID "HISTTOOL1"

#define HT_OK 0
#define HT_ERROR 1

#define DEFAULT_BIN_MAX 10.
#define DEFAULT_BIN_MIN 0.
#define DEFAULT_UPDATE_PERIOD 200   // ms
#define DEFAULT_NBINS 10

#define LOG10_AXIS 100
#define LINEAR_AXIS 101
#define CHANNEL_RANGE 102
#define USER_SPECIFIED_RANGE 103
#define AUTO_SCALE_RANGE 104

#define POLL_INTERVAL 100     // ms

#define ECA_NAMESIZE 34

#include "histToolVersion.h"

// The following must be 1024 or less for WIN32
#define FIXED_MSG_SIZE 1024

#ifdef WIN32
# define DIR_DELIMITER_CHAR '\\'
# define DIR_DELIMITER_STRING "\\"
// Hummingbird extra functions including lprintf
//   Needs to be included after Intrinsic.h for Exceed 5
# include <stdio.h>
# include <X11/Intrinsic.h>
# include <X11/XlibXtra.h>
// The following is done in Exceed 6 but not in Exceed 5
//   Need it to define printf as lprintf for Windows
//   (as opposed to Console) apps */
# ifdef _WINDOWS
#  ifndef printf
#   define printf lprintf
#  endif    
# endif    
#else // #ifdef WIN32
// WIN32 does not have unistd.h
# define DIR_DELIMITER_CHAR '/'
# define DIR_DELIMITER_STRING "/"
# include <stdio.h>
# include <unistd.h>
# include <limits.h>
#endif // #ifdef WIN32

#include <sys/stat.h>
#include <sys/types.h>
//#include <sys/utsname.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

// PATH_MAX
#ifdef WIN32
// Is in stdlib.h for WIN32
#define PATH_MAX _MAX_PATH
#endif

// Unreferenced
#ifdef __cplusplus
#define UNREFERENCED(x) (x)
#else
#define UNREFERENCED(x)
#endif

#include <Xm/Xm.h>

#include "CHist.h"
#include "COptions.h"

#define MAX(x,y) (((x)>=(y))?(x):(y))
#define MIN(x,y) (((x)<=(y))?(x):(y))

// Function prototypes
void optionMenuSet(Widget menu, int buttonId);
int yesNo(char *text, char *ok, char *cancel);

// Global variables

// EXTERN will be defined as blank where the storage is to be allocated
// Otherwise they are declared to be extern
#ifndef EXTERN
#define EXTERN extern
#endif

EXTERN int (*errHandler)(const char *fmt, ...);
EXTERN Display *display;
EXTERN int screen;
EXTERN int nplanes;
EXTERN Screen *screenptr;
EXTERN XtAppContext appContext;
EXTERN Widget appMainWindow;
EXTERN Widget optS;
EXTERN tsDLList<CHist> *windowList;

#define MAXARGS 75
EXTERN Arg args[MAXARGS];
EXTERN Cardinal nargs;

EXTERN COptions options;

#endif     // #ifndef _INCLUDE_HISTTOOL_H
